# -*- coding: utf-8 -*-

from ..core.lang import add_lang

add_lang('zh', {
    'buy': '买入',
    'sell': '卖出',
    'stop loss': '止损',
    'stop earn': '止盈',
    'signal': '信号',
    'You can buy': '建议买入',
    'You can sell': '建议卖出',
    'price': '价格',
    'volume': '成交量',
    'trade': '交易',
    'position': '持仓',
    'return(%)': '收益率(%)',
    'position(%)': '仓位(%)',
    'drawdown(%)': '回撤(%)',
    'quantitative backtesting': '量化回测',
})
