# -*- coding: utf-8 -*-

from ..core.lang import add_lang

add_lang('en', {
    'buy': 'buy',
    'sell': 'sell',
    'stop loss': 'stop loss',
    'stop earn': 'stop earn',
    'signal': 'signal',
    'You can buy': 'You can buy',
    'You can sell': 'You can sell',
    'price': 'price',
    'volume': 'volume',
    'trade': 'trade',
    'position': 'position',
    'return(%)': 'return(%)',
    'position(%)': 'position(%)',
    'drawdown(%)': 'drawdown(%)',
    'quantitative backtesting': 'Quantitative backtesting',
})
